@@ -1,6 +1,6 @@
 import cgi
 import logging
-import os
+import sys
 import re
 from signal import signal, SIGINT, SIGTERM, SIGABRT
 from threading import Thread, Lock
@@ -15,22 +15,26 @@ from requests import Request
 from requests.adapters import HTTPAdapter
 
 from . import __version__ as version
-from .dispatcher import Dispatcher, StopDispatching
-from .event import Event, EventType
-from .filter import Filter
-from .handler import MessageHandler
+from .dispatcher import Dispatcher
+from .event import Event
+from .handler import *
 from .util import signal_name_by_code
 from .myteam import add_chat_members, create_chat
+from .types import InlineKeyboardMarkup
 
-try:
-    from urllib import parse as urlparse
-except ImportError:
-    # noinspection PyUnresolvedReferences
-    import urlparse
+
+def keyboard_to_json(keyboard_markup):
+    if isinstance(keyboard_markup, InlineKeyboardMarkup):
+        return keyboard_markup.to_json()
+    elif isinstance(keyboard_markup, list):
+        return json.dumps(keyboard_markup)
+    else:
+        return keyboard_markup
 
 
 class Bot(object):
-    def __init__(self, token, api_url_base=None, name=None, version=None, timeout_s=20, poll_time_s=60, is_myteam=False):
+    def __init__(self, token: str, api_url_base: str = None, name: str = None, version: str = None,
+                 timeout_s: int = 20, poll_time_s: int = 60, is_myteam: bool = False):
         super(Bot, self).__init__()
 
         self.log = logging.getLogger(__name__)
@@ -51,13 +55,12 @@ class Bot(object):
 
         self.__lock = Lock()
         self.__polling_thread = None
-
         self.__sent_im_cache = ExpiringDict(max_len=2 ** 10, max_age_seconds=60)
         self.dispatcher.add_handler(SkipDuplicateMessageHandler(self.__sent_im_cache))
 
         if self.is_myteam:
-            self.add_chat_members = types.MethodType( add_chat_members, self )
-            self.create_chat = types.MethodType( create_chat, self )
+            self.add_chat_members = types.MethodType(add_chat_members, self)
+            self.create_chat = types.MethodType(create_chat, self)
 
     @property
     def uin(self):
@@ -65,9 +68,13 @@ class Bot(object):
 
     @cached_property
     def user_agent(self):
+
         return "{name}/{version} (uin={uin}) bot-python/{library_version}".format(
-            name=self.name,
-            version=self.version,
+            name=self.name if self.name is not None else requests.get(url="{}/self/get".format(self.api_base_url),
+                                                                      params={"token": self.token}
+                                                                      ).json().get('nick'),
+            # name=self.name,
+            version=self.version if self.version is not None else 'base',
             uin="" if self.uin is None else self.uin,
             library_version=version
         )
@@ -100,7 +107,6 @@ class Bot(object):
             except Exception as e:
                 self.log.exception("Exception while polling: {e}".format(e=e))
 
-
     def start_polling(self):
         with self.__lock:
             if not self.running:
@@ -121,7 +127,7 @@ class Bot(object):
                 self.__polling_thread.join()
 
     # noinspection PyUnusedLocal
-    def _signal_handler(self, sig, stack_frame):
+    def _signal_handler(self, sig: int):
         if self.running:
             self.log.debug("Stopping bot by signal '{name} ({code})'. Repeat for force exit.".format(
                 name=signal_name_by_code(sig), code=sig
@@ -131,7 +137,7 @@ class Bot(object):
             self.log.warning("Force exiting.")
             # It's fine here, this is standard way to force exit.
             # noinspection PyProtectedMember
-            os._exit(1)
+            sys.exit(1)
 
     def idle(self):
         for sig in (SIGINT, SIGTERM, SIGABRT):
@@ -140,7 +146,7 @@ class Bot(object):
         while self.running:
             sleep(1)
 
-    def events_get(self, poll_time_s=None, last_event_id=None):
+    def events_get(self, poll_time_s: int = None, last_event_id: int = None):
         poll_time_s = self.poll_time_s if poll_time_s is None else poll_time_s
         last_event_id = self.last_event_id if last_event_id is None else last_event_id
 
@@ -168,7 +174,98 @@ class Bot(object):
             timeout=self.timeout_s
         )
 
-    def send_text(self, chat_id, text, reply_msg_id=None, forward_chat_id=None, forward_msg_id=None,
+    def default_handler(self):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=DefaultHandler(callback=handler))
+            return handler
+
+        return decorate
+
+    def new_member_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=NewChatMembersHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def member_left_chat_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=LeftChatMembersHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def pin_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=PinnedMessageHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def unpin_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=UnPinnedMessageHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def message_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=MessageHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def edit_msg_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=EditedMessageHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def delete_msg_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=DeletedMessageHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def command_handler(self, command=None, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=CommandHandler(callback=handler, command=command, filters=filters))
+            return handler
+
+        return decorate
+
+    def help_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=HelpCommandHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def start_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=StartCommandHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def unknown_cmd_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=UnknownCommandHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def button_handler(self, filters=None):
+        def decorate(handler):
+            self.dispatcher.add_handler(handler=BotButtonCommandHandler(callback=handler, filters=filters))
+            return handler
+
+        return decorate
+
+    def send_text(self, chat_id: str, text: str, reply_msg_id=None, forward_chat_id=None, forward_msg_id=None,
                   inline_keyboard_markup=None):
         return self.http_session.get(
             url="{}/messages/sendText".format(self.api_base_url),
@@ -179,7 +276,7 @@ class Bot(object):
                 "replyMsgId": reply_msg_id,
                 "forwardChatId": forward_chat_id,
                 "forwardMsgId": forward_msg_id,
-                "inlineKeyboardMarkup": inline_keyboard_markup
+                "inlineKeyboardMarkup": keyboard_to_json(inline_keyboard_markup)
             },
             timeout=self.timeout_s
         )
@@ -197,7 +294,7 @@ class Bot(object):
                 "replyMsgId": reply_msg_id,
                 "forwardChatId": forward_chat_id,
                 "forwardMsgId": forward_msg_id,
-                "inlineKeyboardMarkup": inline_keyboard_markup
+                "inlineKeyboardMarkup": keyboard_to_json(inline_keyboard_markup)
             }
         )
         if file:
@@ -218,7 +315,7 @@ class Bot(object):
                 "replyMsgId": reply_msg_id,
                 "forwardChatId": forward_chat_id,
                 "forwardMsgId": forward_msg_id,
-                "inlineKeyboardMarkup": inline_keyboard_markup
+                "inlineKeyboardMarkup": keyboard_to_json(inline_keyboard_markup)
             }
         )
 
@@ -236,7 +333,7 @@ class Bot(object):
                 "chatId": chat_id,
                 "msgId": msg_id,
                 "text": text,
-                "inlineKeyboardMarkup": inline_keyboard_markup
+                "inlineKeyboardMarkup": keyboard_to_json(inline_keyboard_markup)
             },
             timeout=self.timeout_s
         )
@@ -516,4 +613,3 @@ class SkipDuplicateMessageHandler(MessageHandler):
 
 class InvalidToken(Exception):
     pass
-
